﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace NodeResizing
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            DataContext = TreeData.GenerateTreeData();
        }

        double nodeInitialSize;
        Point startDragPoint;
        private void Rectangle_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            (sender as Rectangle).CaptureMouse();
            startDragPoint = e.GetPosition(this);
            nodeInitialSize = ((sender as Rectangle).Parent as Grid).ActualWidth;
        }

        private void Rectangle_PreviewMouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            (sender as Rectangle).ReleaseMouseCapture();
            nodeInitialSize = 0;
            startDragPoint = new Point();
        }

        private void Rectangle_PreviewMouseMove(object sender, MouseEventArgs e)
        {
            if (e.LeftButton == MouseButtonState.Pressed)
            {
                Point mousePoint = e.GetPosition(this);

                ((sender as Rectangle).Parent as Grid).Width = nodeInitialSize + (mousePoint.X - startDragPoint.X);
            }
        }
    }

    public class TreeData : PropertyChangedNotifier
    {
        private int id;

        public int ID
        {
            get { return id; }
            set { id = value; OnPropertyChagned("ID"); }
        }

        private string name;

        public string Name
        {
            get { return name; }
            set { name = value; OnPropertyChagned("Name"); }
        }

        private ObservableCollection<TreeData> children;

        public ObservableCollection<TreeData> Children
        {
            get { return children; }
            set { children = value; OnPropertyChagned("Children"); }
        }

        public static ObservableCollection<TreeData> GenerateTreeData()
        {
            ObservableCollection<TreeData> data = new ObservableCollection<TreeData>();

            for (int i = 0; i < 10; i++)
            {
                TreeData n = new TreeData
                {
                    ID = i,
                    Name = "Item " + i,
                    Children = new ObservableCollection<TreeData>()
                };

                for (int j = 0; j < 5; j++)
                {
                    TreeData n1 = new TreeData
                    {
                        ID = i + j,
                        Name = "Item " + i + " " + j,
                        Children = new ObservableCollection<TreeData>()
                    };

                    for (int p = 0; p < 5; p++)
                    {
                        n1.children.Add(new TreeData
                        {
                            ID = i + j + p,
                            Name = "Item " + i + " " + j + " " + p,
                        });
                    }

                    n.Children.Add(n1);
                }

                data.Add(n);
            }

            return data;
        }

    }
     

    public class PropertyChangedNotifier : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChagned(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }
}
